;; Tufte/theme.jl

;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>

;; This file is part of sawmill.

;; sawmill is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; sawmill is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with sawmill; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

(defgroup Tufte "Tufte theme"
	:group appearance)

;; PROPS: derived mostly from Simple theme, with GTK color cloning from
;; GTK theme.  Wacky, inconsistent, indentation is all me, however, as
;; I attempted to wrap my head around creating readable Lisp.

;; TODO: compute decoration and button dimensions based on screen dimensions
;; to maintain usable sizes at all resolutions.  Can maintain discreet images
;; for different sizes, and scale for intermediate values.

(let*
	(
		;; buttons

		;; TODO: mechanism for using GTK foreground for button foreground 
		;; 		color, including the different states: normal, focused,
		;;		highlighted, or clicked 

		(minimize `((inactive . ,(make-image "min.png"))
			(clicked . ,(make-image "min-c.png"))))
		(close `((inactive . ,(make-image "close.png"))
			(clicked . ,(make-image "close-c.png"))))
		(maximize `((inactive . ,(make-image "max.png"))
			(clicked . ,(make-image "max-c.png"))))
		(restore `((inactive . ,(make-image "restore.png"))
			(clicked . ,(make-image "restore-c.png"))))
		(maximize-restore (lambda (w)
				(if (window-maximized-p w) restore maximize)))
		(shade `((inactive . ,(make-image "shade.png"))
			(clicked . ,(make-image "shade-c.png"))))
		(unshade `((inactive . ,(make-image "unshade.png"))
			(clicked . ,(make-image "unshade-c.png"))))
		(shade-restore (lambda (w)
				(if (window-get w 'shaded) unshade shade)))
		(menu `((inactive . ,(make-image "menu.png"))
			(clicked . ,(make-image "menu-c.png"))))

		(initialised-gtk nil)

		(rebuild (lambda ()
			(when (not initialised-gtk)
				(setq initialised-gtk t)
				(require 'gtkrc)
				(gtkrc-call-after-changed (lambda () 
					(rebuild-frames-with-style 'Tufte)
				))
			)
			(rebuild-frames-with-style 'Tufte)
		))

		(foreground (lambda () gtkrc-foreground))
		(background (lambda () gtkrc-background))
		(font (lambda () gtkrc-font))

		(frame `(
			;; title bar
			((background . ,background)
				(foreground . ,foreground)

				;;	TODO: pad text in titlebar so long names or names in 
				;;		narrow windows aren't flush against left border and 
				;;		right buttons

				(text . ,window-name)
				(font . ,font)
				(x-justify . center)
				(y-justify . center)
				(left-edge . 0)
				(right-edge . 63)
				(top-edge . -21)
				(height . 21)
				(class . title))
			((background . ,foreground)
				(left-edge . 0)
				(right-edge . 0)
				(top-edge . -22)
				(height . 1))
			((background . ,foreground)
				(left-edge . -1)
				(width . 1)
				(top-edge . -22)
				(bottom-edge . -11))
			((background . ,foreground)
				(right-edge . -1)
				(width . 1)
				(top-edge . -22)
				(bottom-edge . -11))
			((background . ,foreground)
				(left-edge . 0)
				(right-edge . 0)
				(top-edge . 0)
				(height . 1))

			;; buttons

			;; TODO: support "removable" buttons by having a greyed out version
			;;		beneath the removable button.  Only the "normal" background
			;;		color should be specified, no highlighting or clicked
			;;		coloration.

			;;((background . ,background)
			;;	(foreground . ,menu)
			;;	(left-edge . 0)
			;;	(top-edge . -21)
			;;	(width . 21)
			;;	(height . 21)
			;;	(class . menu-button))
			((background . ,background)
				(foreground . ,minimize)
				(right-edge . 42)
				(top-edge . -21)
				(width . 21)
				(height . 21)
				(class . iconify-button))
			;;((background . ,background)
			;;	(foreground . ,shade-restore)
			;;	(right-edge . 42)
			;;	(top-edge . -21)
			;;	(width . 21)
			;;	(height . 21)
			;;	(class . shade-button))
			((background . ,background)
				(foreground . ,maximize-restore)
				(right-edge . 21)
				(top-edge . -21)
				(width . 21)
				(height . 21)
				(class . maximize-button))
			((background . ,background)
				(foreground . ,close)
				(right-edge . 0)
				(top-edge . -21)
				(width . 21)
				(height . 21)
				(class . close-button))

			;; bottom bar
			;; bottom-left-corner
			((background . ,background)
				(left-edge . 0)
				(width . 21)
				(bottom-edge . -11)
				(height . 10)
				(class . bottom-left-corner))
			((background . ,foreground)
				(left-edge . 21)
				(width . 1)
				(bottom-edge . -11)
				(height . 10))

			 ;; bottom-border
			((background . ,foreground)
				(left-edge . 0)
				(right-edge . 0)
				(bottom-edge . -1)
				(height . 1))
			((background . ,background)
				(left-edge . 22)
				(right-edge . 22)
				(bottom-edge . -11)
				(height . 10)
				(class . bottom-border))

			 ;; bottom-right-corner
			((background . ,foreground)
				(right-edge . 21)
				(width . 1)
				(bottom-edge . -11)
				(height . 10))
			((background . ,background)
				(right-edge . 0)
				(width . 21)
				(bottom-edge . -11)
				(height . 10)
				(class . bottom-right-corner))

			 ;; frame bottom
			((background . ,foreground)
				(left-edge . 0)
				(right-edge . 0)
				(bottom-edge . -11)
				(height . 1))
		))

		(shaped-frame `(
			;; title bar
			((background . ,background)
				(foreground . ,foreground)
				(text . ,window-name)
				(font . ,font)
				(x-justify . -69)
				(y-justify . center)
				(left-edge . 0)
				(right-edge . 0)
				(top-edge . -21)
				(height . 21)
				(class . title))
			((background . ,foreground)
				(left-edge . 0)
				(right-edge . 0)
				(top-edge . -22)
				(height . 1))
			((background . ,foreground)
				(left-edge . -1)
				(width . 1)
				(top-edge . -22)
				(height . 23))
			((background . ,foreground)
				(right-edge . -1)
				(width . 1)
				(top-edge . -22)
				(height . 23))
			((background . ,foreground)
				 (left-edge . 0)
				 (right-edge . 0)
				 (top-edge . 0)
				 (height . 1))

			;; buttons
			;;((background . ,background)
			;;	(foreground . ,menu)
			;;	(left-edge . 0)
			;;	(top-edge . -21)
			;;	(width . 21)
			;;	(height . 21)
			;;	(class . menu-button))
			((background . ,background)
				(foreground . ,minimize)
				(right-edge . 42)
				(top-edge . -21)
				(width . 21)
				(height . 21)
				(class . iconify-button))
			;;((background . ,background)
			;;	(foreground . ,shade-restore)
			;;	(right-edge . 42)
			;;	(top-edge . -21)
			;;	(width . 21)
			;;	(height . 21)
			;;	(class . shade-button))
			((background . ,background)
				(foreground . ,maximize-restore)
				(right-edge . 21)
				(top-edge . -21)
				(width . 21)
				(height . 21)
				(class . maximize-button))
			((background . ,background)
				(foreground . ,close)
				(right-edge . 0)
				(top-edge . -21)
				(width . 21)
				(height . 21)
				(class . close-button))
		))

		(transient-frame `(
			;; title bar
			((background . ,background)
				(left-edge . 0)
				(right-edge . 0)
				(top-edge . -11)
				(height . 10)
				(class . title))
			((background . ,foreground)
				(left-edge . 0)
				(right-edge . 0)
				(top-edge . -11)
				(height . 1))
			((background . ,foreground)
				(left-edge . -1)
				(width . 1)
				(top-edge . -11)
				(bottom-edge . -11))
			((background . ,foreground)
				(right-edge . -1)
				(width . 1)
				(top-edge . -11)
				(bottom-edge . -11))
			((background . ,foreground)
				(left-edge . 0)
				(right-edge . 0)
				(top-edge . -1)
				(height . 1))

			;; bottom bar
			;; bottom-left-corner
			((background . ,background)
				(left-edge . 0)
				(width . 21)
				(bottom-edge . -11)
				(height . 10)
				(class . bottom-left-corner))
			((background . ,foreground)
				(left-edge . 21)
				(width . 1)
				(bottom-edge . -11)
				(height . 10))

			 ;; bottom-border
			((background . ,foreground)
				(left-edge . 0)
				(right-edge . 0)
				(bottom-edge . -1)
				(height . 1))
			((background . ,background)
				(left-edge . 22)
				(right-edge . 22)
				(bottom-edge . -11)
				(height . 10)
				(class . bottom-border))

			 ;; bottom-right-corner
			((background . ,foreground)
				(right-edge . 21)
				(width . 1)
				(bottom-edge . -11)
				(height . 10))
			((background . ,background)
				(right-edge . 0)
				(width . 21)
				(bottom-edge . -11)
				(height . 10)
				(class . bottom-right-corner))

			 ;; frame bottom
			((background . ,foreground)
				(left-edge . 0)
				(right-edge . 0)
				(bottom-edge . -11)
				(height . 1))
		))

		(shaped-transient-frame `(
			;; title bar
			((background . ,background)
				(left-edge . 0)
				(right-edge . 0)
				(top-edge . -11)
				(height . 10)
				(class . title))

			((background . ,foreground)
				(left-edge . 0)
				(right-edge . 0)
				(top-edge . -11)
				(height . 1))
			((background . ,foreground)
				(left-edge . -1)
				(width . 1)
				(top-edge . -11)
				(height . 11))
			((background . ,foreground)
				(right-edge . -1)
				(width . 1)
				(top-edge . -11)
				(height . 11))
			((background . ,foreground)
				(left-edge . 0)
				(right-edge . 0)
				(top-edge . -1)
				(height . 1))
		))
	)

	(add-frame-style 'Tufte (lambda (w type)
		(case type ((default) frame)
			((transient) transient-frame)
			((shaped) shaped-frame)
			((shaped-transient) shaped-transient-frame)
		)
	))

	(rebuild)
)
